@@ -38,6 +38,7 @@
 #include <errno.h>
 
 #include <urcu/arch.h>
+#include <urcu/tls-compat.h>
 
 #ifdef __linux__
 #include <syscall.h>
@@ -148,11 +149,11 @@ static int test_duration_enqueue(void)
 	return !test_stop;
 }
 
-static unsigned long long __thread nr_dequeues;
-static unsigned long long __thread nr_enqueues;
+static DEFINE_URCU_TLS(unsigned long long, nr_dequeues);
+static DEFINE_URCU_TLS(unsigned long long, nr_enqueues);
 
-static unsigned long long __thread nr_successful_dequeues;
-static unsigned long long __thread nr_successful_enqueues;
+static DEFINE_URCU_TLS(unsigned long long, nr_successful_dequeues);
+static DEFINE_URCU_TLS(unsigned long long, nr_successful_enqueues);
 
 static unsigned int nr_enqueuers;
 static unsigned int nr_dequeuers;
@@ -187,24 +188,24 @@ void *thr_enqueuer(void *_count)
 		cds_lfs_node_init_rcu(&node->list);
 		/* No rcu read-side is needed for push */
 		cds_lfs_push_rcu(&s, &node->list);
-		nr_successful_enqueues++;
+		URCU_TLS(nr_successful_enqueues)++;
 
 		if (caa_unlikely(wdelay))
 			loop_sleep(wdelay);
 fail:
-		nr_enqueues++;
+		URCU_TLS(nr_enqueues)++;
 		if (caa_unlikely(!test_duration_enqueue()))
 			break;
 	}
 
 	rcu_unregister_thread();
 
-	count[0] = nr_enqueues;
-	count[1] = nr_successful_enqueues;
+	count[0] = URCU_TLS(nr_enqueues);
+	count[1] = URCU_TLS(nr_successful_enqueues);
 	printf_verbose("enqueuer thread_end, thread id : %lx, tid %lu, "
 		       "enqueues %llu successful_enqueues %llu\n",
-		       pthread_self(), (unsigned long)gettid(), nr_enqueues,
-		       nr_successful_enqueues);
+		       pthread_self(), (unsigned long)gettid(),
+		       URCU_TLS(nr_enqueues), URCU_TLS(nr_successful_enqueues));
 	return ((void*)1);
 
 }
@@ -249,9 +250,9 @@ void *thr_dequeuer(void *_count)
 		rcu_read_unlock();
 		if (node) {
 			call_rcu(&node->rcu, free_node_cb);
-			nr_successful_dequeues++;
+			URCU_TLS(nr_successful_dequeues)++;
 		}
-		nr_dequeues++;
+		URCU_TLS(nr_dequeues)++;
 		if (caa_unlikely(!test_duration_dequeue()))
 			break;
 		if (caa_unlikely(rduration))
@@ -263,10 +264,10 @@ void *thr_dequeuer(void *_count)
 
 	printf_verbose("dequeuer thread_end, thread id : %lx, tid %lu, "
 		       "dequeues %llu, successful_dequeues %llu\n",
-		       pthread_self(), (unsigned long)gettid(), nr_dequeues,
-		       nr_successful_dequeues);
-	count[0] = nr_dequeues;
-	count[1] = nr_successful_dequeues;
+		       pthread_self(), (unsigned long)gettid(),
+		       URCU_TLS(nr_dequeues), URCU_TLS(nr_successful_dequeues));
+	count[0] = URCU_TLS(nr_dequeues);
+	count[1] = URCU_TLS(nr_successful_dequeues);
 	return ((void*)2);
 }
 
@@ -379,7 +380,9 @@ int main(int argc, char **argv)
 	count_dequeuer = malloc(2 * sizeof(*count_dequeuer) * nr_dequeuers);
 	cds_lfs_init_rcu(&s);
 	err = create_all_cpu_call_rcu_data(0);
-	assert(!err);
+	if (err) {
+		printf("Per-CPU call_rcu() worker threads unavailable. Using default global worker thread.\n");
+	}
 
 	next_aff = 0;
 
