@@ -8,14 +8,15 @@
  *
  * Copyright (c) 2009 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
  *
- * THIS MATERIAL IS PROVIDED AS IS, WITH ABSOLUTELY NO WARRANTY EXPRESSED
- * OR IMPLIED.  ANY USE IS AT YOUR OWN RISK.
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
  *
- * Permission is hereby granted to use or copy this program
- * for any purpose,  provided the above notices are retained on all copies.
- * Permission to modify the code and to distribute modified code is granted,
- * provided the above notices are retained, and a notice that the code was
- * modified is included with the above copyright notice.
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
  */
 
 #include <stddef.h>	/* for offsetof */
@@ -85,4 +86,9 @@
 #define URCU_FORCE_CAST(type, arg)	((type) (arg))
 #endif
 
+#define caa_is_signed_type(type)	(((type) (-1)) < 0)
+
+#define caa_cast_long_keep_sign(v)	\
+	(caa_is_signed_type(__typeof__(v)) ? (long) (v) : (unsigned long) (v))
+
 #endif /* _URCU_COMPILER_H */
