@@ -39,16 +39,17 @@ struct cds_lfs_stack_rcu {
 
 #include <urcu/static/rculfstack.h>
 
-#define cds_lfs_node_init_rcu	_cds_lfs_node_init_rcu
-#define cds_lfs_init_rcu	_cds_lfs_init_rcu
-#define cds_lfs_push_rcu	_cds_lfs_push_rcu
-#define cds_lfs_pop_rcu		_cds_lfs_pop_rcu
+#define cds_lfs_node_init_rcu		_cds_lfs_node_init_rcu
+#define cds_lfs_init_rcu		_cds_lfs_init_rcu
+#define cds_lfs_push_rcu		_cds_lfs_push_rcu
+#define cds_lfs_pop_rcu			_cds_lfs_pop_rcu
 
 #else /* !_LGPL_SOURCE */
 
 extern void cds_lfs_node_init_rcu(struct cds_lfs_node_rcu *node);
 extern void cds_lfs_init_rcu(struct cds_lfs_stack_rcu *s);
-extern void cds_lfs_push_rcu(struct cds_lfs_stack_rcu *s, struct cds_lfs_node_rcu *node);
+extern int cds_lfs_push_rcu(struct cds_lfs_stack_rcu *s,
+			struct cds_lfs_node_rcu *node);
 
 /*
  * Should be called under rcu read lock critical section.
